c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine debmss (nat,x,mass,tmass,cc)
C----
C---- returns coordinates of center of mass in cc
C----
C---- written by Daan van Aalten en Bert de Groot
C---- 
C----
C---- nat    : number of atoms 
C---- x      : atomic coordinates 
C---- mass   : atomic masses      
C---- tmass  : total mass         
C---- cc     : delivered with coordinates of center of mass 
C----
      implicit none

      integer nat,cnt1,cnt2,i3
      real*4  x(*),mass(*),tmass,cc(3)
C----
C---- initialize coordinates of center of mass
C----
      cc(1)=0.0
      cc(2)=0.0
      cc(3)=0.0
C----
C---- mulitply coordinates with masses
C----
      do cnt1=1,nat
         i3=3*(cnt1-1)
         do cnt2=1,3
            cc(cnt2)=cc(cnt2)+x(i3+cnt2)*mass(cnt1)
         enddo
      enddo
C----
C---- divide by tmass to obtain center of mass coordinates
C----
      do cnt1=1,3
         cc(cnt1)=cc(cnt1)/tmass
      enddo

      return
      end

      logical function newres(i,nrs)
C----
C---- becomes true if atom i is the first atom of this resdue
C----
      implicit none
      integer i,nrs(*)

      newres=.false.

      if (i.eq.1) then
         newres=.true.
         return
      endif

      if (nrs(i).eq.nrs(i-1)+1) newres=.true.

      return

      end

      integer function nextres(n,resnr)
C----
C---- returns atom number of first atom in residue subsequent to
C---- the residue that contains n
C----
      implicit none
      integer n,i,resnr(*)

      do i=1,50
         if (resnr(i+n).ne.resnr(n)) then
            nextres=i+n
            return
         endif
      enddo

      write(0,*) 'ERROR IN NEXTRES'

      stop

      end

      subroutine defimp(nat,resnr,resnam,atnam,iimp,nimp)
C----
C---- define improper dihedrals in terms of atom numbers
C----
      implicit none
      include 'params.f'
      integer i,nat,nimp,iimp(4,maxat),resnr(maxat),findatom2
      character*5 resnam(maxat),atnam(maxat)
      logical newres

      nimp=0
      do i=1,nat
         if (newres(i,resnr)) then
C----
C---- N terminal hydrogens
C----
            if (resnr(i).eq.1) then
               if (findatom2(i,atnam,resnr,'   H1',nat).ne.0) then
                  nimp=nimp+1
                  iimp(1,nimp)=findatom2(i,atnam,resnr,'    N',nat)
                  iimp(2,nimp)=findatom2(i,atnam,resnr,'   CA',nat)
                  iimp(3,nimp)=findatom2(i,atnam,resnr,'   H2',nat)
                  iimp(4,nimp)=findatom2(i,atnam,resnr,'   H1',nat)
                  if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &                 iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) 
     &                 nimp=nimp-1
                  nimp=nimp+1
                  iimp(1,nimp)=findatom2(i,atnam,resnr,'    N',nat)
                  iimp(2,nimp)=findatom2(i,atnam,resnr,'   CA',nat)
                  iimp(3,nimp)=findatom2(i,atnam,resnr,'   H1',nat)
                  iimp(4,nimp)=findatom2(i,atnam,resnr,'   H3',nat)
                  if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &                 iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) 
     &                 nimp=nimp-1
               endif
            endif
C----
C---- make sure we have l- amino acids
C----
            if (resnam(i).ne.'GLY  ') then
               if (findatom2(i,atnam,resnr,'   CA',nat).ne.0) then
                  nimp=nimp+1
                  iimp(1,nimp)=findatom2(i,atnam,resnr,'   CA',nat)
                  iimp(2,nimp)=findatom2(i,atnam,resnr,'    N',nat)
                  iimp(3,nimp)=findatom2(i,atnam,resnr,'    C',nat)
                  iimp(4,nimp)=findatom2(i,atnam,resnr,'   CB',nat)
                  if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &                 iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) 
     &                 nimp=nimp-1
               endif
            endif
C----
C---- side chains
C----
            if (resnam(i).eq.'LYSH ') then
               if (findatom2(i,atnam,resnr,'  HZ1',nat).ne.0) then
                  nimp=nimp+1
                  iimp(1,nimp)=findatom2(i,atnam,resnr,'   NZ',nat)
                  iimp(2,nimp)=findatom2(i,atnam,resnr,'   CE',nat)
                  iimp(3,nimp)=findatom2(i,atnam,resnr,'  HZ2',nat)
                  iimp(4,nimp)=findatom2(i,atnam,resnr,'  HZ1',nat)
                  if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &                 iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) 
     &                 nimp=nimp-1
                  nimp=nimp+1
                  iimp(1,nimp)=findatom2(i,atnam,resnr,'   NZ',nat)
                  iimp(2,nimp)=findatom2(i,atnam,resnr,'   CE',nat)
                  iimp(3,nimp)=findatom2(i,atnam,resnr,'  HZ1',nat)
                  iimp(4,nimp)=findatom2(i,atnam,resnr,'  HZ3',nat)
                  if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &                 iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) 
     &                 nimp=nimp-1
               endif
            endif
            if (resnam(i).eq.'LEU  ') then
               nimp=nimp+1
               iimp(1,nimp)=findatom2(i,atnam,resnr,'   CG',nat)
               iimp(2,nimp)=findatom2(i,atnam,resnr,'  CD2',nat)
               iimp(3,nimp)=findatom2(i,atnam,resnr,'  CD1',nat)
               iimp(4,nimp)=findatom2(i,atnam,resnr,'   CB',nat)
               if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &              iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) nimp=nimp-1
            endif
            if (resnam(i).eq.'VAL  ') then
               nimp=nimp+1
               iimp(1,nimp)=findatom2(i,atnam,resnr,'   CB',nat)
               iimp(2,nimp)=findatom2(i,atnam,resnr,'  CG2',nat)
               iimp(3,nimp)=findatom2(i,atnam,resnr,'  CG1',nat)
               iimp(4,nimp)=findatom2(i,atnam,resnr,'   CA',nat)
               if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &              iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) nimp=nimp-1
            endif
            if (resnam(i).eq.'ILE  ') then
               nimp=nimp+1
               iimp(1,nimp)=findatom2(i,atnam,resnr,'   CB',nat)
               iimp(2,nimp)=findatom2(i,atnam,resnr,'  CG1',nat)
               iimp(3,nimp)=findatom2(i,atnam,resnr,'  CG2',nat)
               iimp(4,nimp)=findatom2(i,atnam,resnr,'   CA',nat)
               if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &              iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) nimp=nimp-1
            endif
            if (resnam(i).eq.'THR  ') then
               nimp=nimp+1
               iimp(1,nimp)=findatom2(i,atnam,resnr,'   CB',nat)
               iimp(2,nimp)=findatom2(i,atnam,resnr,'  OG1',nat)
               iimp(3,nimp)=findatom2(i,atnam,resnr,'  CG2',nat)
               iimp(4,nimp)=findatom2(i,atnam,resnr,'   CA',nat)
               if (iimp(1,nimp).eq.0.or.iimp(2,nimp).eq.0.or.
     &              iimp(3,nimp).eq.0.or.iimp(4,nimp).eq.0) nimp=nimp-1
            endif
         endif
      enddo

      return

      end
      integer function lastatom(n,resnr,nat)
C----
C---- returns atom number of last atom in same residue as atom n
C----
      implicit none
      integer n,i,j,nat,resnr(*)

      lastatom=n
      j=min(nat,n+50)
      do i=n,j
         if (resnr(i).eq.resnr(n)) lastatom=i
      enddo

      return

      end
      integer function firstatom(n,resnr)
C----
C---- returns atom number of first atom in same residue as atom n
C----
      implicit none
      integer n,i,j,resnr(*)

      firstatom=1
      j=max(1,n-50)
      do i=n,j,-1
         if (resnr(i).eq.resnr(n)) firstatom=i
      enddo

      return

      end
      real function getlenxx(at1,at2,x)
C----
C---- calculates squared distance at1 at2
C----

      implicit      none
      include 'params.f'
      integer       at1, at2
      real*4        x(3,maxat),vecx,vecy,vecz

      vecx=x(1,at1)-x(1,at2)
      vecy=x(2,at1)-x(2,at2)
      vecz=x(3,at1)-x(3,at2)

      getlenxx=sqrt(vecx*vecx+vecy*vecy+vecz*vecz)
      return
      end
      subroutine getimpxx(nimp,iimp,x,imp)
C----
C---- calculate a number of dihedrals
C----
      implicit none
      include 'params.f'
      integer i,nimp,iimp(4,*)
      real*4 x(3,maxat),imp(*),gettaxx

      do i=1,nimp
         imp(i)=gettaxx(iimp(1,i),iimp(2,i),iimp(3,i),iimp(4,i),x)
      enddo

      return

      end
      real function gettaxx(at1,at2,at3,at4,x)
C----
C----
C---- calculates torsion angle at1 at2 at3 at4
C----
C----
      implicit      none
      include 'params.f'
      integer       at1, at2, at3, at4
      real*4        x(3,maxat), ux,uy,uz,
     +              vx,vy,vz,wx,wy,wz,uvx,uvy,uvz,vwx,vwy,vwz,
     +              uvl,vwl,cosphi,temp,si,uvlvwl

         ux = x(1,at2) - x(1,at1)
         uy = x(2,at2) - x(2,at1)
         uz = x(3,at2) - x(3,at1)
         vx = x(1,at3) - x(1,at2)
         vy = x(2,at3) - x(2,at2)
         vz = x(3,at3) - x(3,at2)
         wx = x(1,at4) - x(1,at3)
         wy = x(2,at4) - x(2,at3)
         wz = x(3,at4) - x(3,at3)

         uvx = uy*vz - uz*vy
         uvy = uz*vx - ux*vz
         uvz = ux*vy - uy*vx

         vwx = vy*wz - vz*wy
         vwy = vz*wx - vx*wz
         vwz = vx*wy - vy*wx

         uvl = sqrt(uvx*uvx + uvy*uvy + uvz*uvz)
         vwl = sqrt(vwx*vwx + vwy*vwy + vwz*vwz)

         uvlvwl=uvl*vwl
         if (uvlvwl.eq.0.000000) uvlvwl=1e-12
         cosphi = (uvx*vwx + uvy*vwy + uvz*vwz) / uvlvwl
         if (cosphi.gt.1.000000) cosphi=1.000000
         if (cosphi.lt.-1.000000) cosphi=-1.000000
         temp=uvx*wx+uvy*wy+uvz*wz
         if (abs(temp).eq.0.000000) then
            si=1.0
         else
            si=temp/abs(temp)
         end if
         gettaxx = si*(360.0 / (2.0*3.1415927))*acos(cosphi)

      return
      end
      real function getta(at1,at2,at3,at4,x)
C----
C----
C---- calculates torsion angle at1 at2 at3 at4
C----
C----
      implicit      none
      integer       at1, at2, at3, at4
      real*4        x(*), ux,uy,uz,
     +              vx,vy,vz,wx,wy,wz,uvx,uvy,uvz,vwx,vwy,vwz,
     +              uvl,vwl,cosphi,temp,si,uvlvwl

      ux = x(3*at2-2) - x(3*at1-2)
      uy = x(3*at2-1) - x(3*at1-1)
      uz = x(3*at2)   - x(3*at1)
      vx = x(3*at3-2) - x(3*at2-2)
      vy = x(3*at3-1) - x(3*at2-1)
      vz = x(3*at3)   - x(3*at2)
      wx = x(3*at4-2) - x(3*at3-2)
      wy = x(3*at4-1) - x(3*at3-1)
      wz = x(3*at4)   - x(3*at3)
      
      uvx = uy*vz - uz*vy
      uvy = uz*vx - ux*vz
      uvz = ux*vy - uy*vx

      vwx = vy*wz - vz*wy
      vwy = vz*wx - vx*wz
      vwz = vx*wy - vy*wx
      
      uvl = sqrt(uvx*uvx + uvy*uvy + uvz*uvz)
      vwl = sqrt(vwx*vwx + vwy*vwy + vwz*vwz)
      
      uvlvwl=uvl*vwl
      if (uvlvwl.eq.0.000000) uvlvwl=1e-12
      cosphi = (uvx*vwx + uvy*vwy + uvz*vwz) / uvlvwl
      if (cosphi.gt.1.000000) cosphi=1.000000
      if (cosphi.lt.-1.000000) cosphi=-1.000000
      temp=uvx*wx+uvy*wy+uvz*wz
      if (abs(temp).eq.0.000000) then
         si=1.0
      else
         si=temp/abs(temp)
      end if
      getta = si*(360.0 / (2.0*3.1415927))*acos(cosphi)

      return
      end

      subroutine swapxx(n,resnr,resnam,atnam,x,nat,viol)
C----
C---- generic swap routine for atoms involved in incorrect
C---- improper dihedrals
C----
      implicit none
      include 'params.f'
      integer n,j,k,n1,n2,n3,n4,findatom2,lastatom,resnr(*),nat
      real*4 x(3,maxat)
      character*5 resnam(*),atnam(*)
      logical viol(*)
C----
C---- swap of n-terminal hydrogens
C----
      j=0
      k=0
      if (atnam(n).eq.'   H2') then
         j=findatom2(n,atnam,resnr,'   H1',nat)
         k=findatom2(n,atnam,resnr,'   H2',nat)
         call swapitxx(j,k,x)
      else if (atnam(n).eq.'   H1') then
         j=findatom2(n,atnam,resnr,'   H1',nat)
         k=findatom2(n,atnam,resnr,'   H3',nat)
         call swapitxx(j,k,x)
C----
C---- conversion of d to l amino acid
C----
      else if (atnam(n).eq.'    C') then
         n1=findatom2(n,atnam,resnr,'   CA',nat)
         n2=findatom2(n,atnam,resnr,'    N',nat)
         n3=findatom2(n,atnam,resnr,'    C',nat)
         k=lastatom(n,resnr,nat)
         do n4=n1+1,k
            viol(n4)=.true.
            if ((atnam(n4).ne.'    C').and.(atnam(n4).ne.'    O'))
     &           call mirrorxx(n1,n2,n3,n4,x)
         enddo
C----
C---- swap of hydrogens in lysh nh3
C----
      else if (resnam(n).eq.'LYSH ') then
         if (atnam(n).eq.'  HZ2') then
            j=findatom2(n,atnam,resnr,'  HZ1',nat)
            k=findatom2(n,atnam,resnr,'  HZ2',nat)
            call swapitxx(j,k,x)
         else if (atnam(n).eq.'  HZ1') then
            j=findatom2(n,atnam,resnr,'  HZ1',nat)
            k=findatom2(n,atnam,resnr,'  HZ3',nat)
            call swapitxx(j,k,x)
         endif
C----
C---- swap of CD1 and CD2 in LEU or CG1 and CG2 in VAL
C----
      else if (resnam(n).eq.'LEU  ') then
         j=findatom2(n,atnam,resnr,'  CD1',nat)
         k=findatom2(n,atnam,resnr,'  CD2',nat)
         call swapitxx(j,k,x)
      else if (resnam(n).eq.'VAL  ') then
         j=findatom2(n,atnam,resnr,'  CG1',nat)
         k=findatom2(n,atnam,resnr,'  CG2',nat)
         call swapitxx(j,k,x)
C----
C---- mirror ILE or THR side chains
C----
      else if (resnam(n).eq.'ILE  ') then
         n1=findatom2(n,atnam,resnr,'  CG2',nat)
         n2=findatom2(n,atnam,resnr,'   CB',nat)
         n3=findatom2(n,atnam,resnr,'   CA',nat)
         n4=findatom2(n,atnam,resnr,'   CD',nat)
         viol(n4)=.true.
         call mirrorxx(n1,n2,n3,n4,x)
         n1=findatom2(n,atnam,resnr,'  CG2',nat)
         n2=findatom2(n,atnam,resnr,'   CB',nat)
         n3=findatom2(n,atnam,resnr,'   CA',nat)
         n4=findatom2(n,atnam,resnr,'  CG1',nat)
         viol(n4)=.true.
         call mirrorxx(n1,n2,n3,n4,x)
      else if (resnam(n).eq.'THR  ') then
         n1=findatom2(n,atnam,resnr,'  OG1',nat)
         n2=findatom2(n,atnam,resnr,'   CB',nat)
         n3=findatom2(n,atnam,resnr,'   CA',nat)
         n4=findatom2(n,atnam,resnr,'  CG2',nat)
         call mirrorxx(n1,n2,n3,n4,x)
         viol(n4)=.true.
         n4=findatom2(n,atnam,resnr,'  HG1',nat)
         if (n4.ne.0) then
            viol(n4)=.true.
            call mirrorxx(n1,n2,n3,n4,x)
         endif
      else
         write (0,*) 'WRONG RESIDUE NAME IN SWAPX:',n,resnam(n),atnam(n)
         stop
      endif

      if (j.ne.0) viol(j)=.true.
      if (k.ne.0) viol(k)=.true.

      return

      end
      subroutine swapitxx(n1,n2,x)
C----
C---- swaps atom n1 and atom n2 in x
C----
      implicit none
      include 'params.f'
      integer n1,n2,i
      real*4 x(3,maxat),dum(3)

      do i=1,3
         dum(i)=x(i,n1)
         x(i,n1)=x(i,n2)
         x(i,n2)=dum(i)
      enddo

      return

      end
      subroutine mirrorxx(n1,n2,n3,n4,x)
C----
C---- adapt the coordinates of atom n4 such that it is mirrorred
C---- with respect to the plane defined by n1,n2,n3 (n1 origin)
C----
      implicit none
      include 'params.f'
      integer n1,n2,n3,n4
      real*4 x(3,maxat),v1(3),v2(3),v3(3),v4(3),a1,a2,rk,rl,
     &     l1,l2,i12,o(3),p(3),pp(3),getlenxx,clen,p3,p4
#     if defined(DEBUG)
      real l3,l4,ip
#     endif
C----
C---- initialize
C----
      o(1)=x(1,n1)
      o(2)=x(2,n1)
      o(3)=x(3,n1)

      p(1)=x(1,n4)
      p(2)=x(2,n4)
      p(3)=x(3,n4)
C----
C---- v1 and v2 span the plane
C----
      v1(1)=x(1,n2)-x(1,n1)
      v1(2)=x(2,n2)-x(2,n1)
      v1(3)=x(3,n2)-x(3,n1)

      v2(1)=x(1,n3)-x(1,n1)
      v2(2)=x(2,n3)-x(2,n1)
      v2(3)=x(3,n3)-x(3,n1)

      l1=getlenxx(n1,n2,x)
      l2=getlenxx(n1,n3,x)
      clen=getlenxx(n1,n4,x)
      i12=v1(1)*v2(1)+v1(2)*v2(2)+v1(3)*v2(3)
C----
C---- we need orthonormal basis set
C----
C---- l3 is normalized l1
C---- l4 is orthogonal to it and lies in the plane
C---- (linear combination of l1 and l2, with a1 and a2 as coefficients)
C----
      rk=0.0-i12/(l1**2)
      rl=(rk**2)*(l1**2)+(l2**2)+2*rk*i12

      a2=sqrt(rl)/rl
      a1=rk*a2

      v3(1)=v1(1)/l1
      v3(2)=v1(2)/l1
      v3(3)=v1(3)/l1

      v4(1)=a1*v1(1)+a2*v2(1)
      v4(2)=a1*v1(2)+a2*v2(2)
      v4(3)=a1*v1(3)+a2*v2(3)
C----
C---- bug check
C----
#     if defined(DEBUG)
      l3=sqrt(v3(1)**2+v3(2)**2+v3(3)**2)
      l4=sqrt(v4(1)**2+v4(2)**2+v4(3)**2)
      ip=v3(1)*v4(1)+v3(2)*v4(2)+v3(3)*v4(3)
      if (abs(1.0-l3).gt.1e-3) 
     &   write (0,*) 'warning : length of V3 in subroutine mirror:',l3
      if (abs(1.0-l4).gt.1e-3)
     &   write (0,*) 'warning : length of V4 in subroutine mirror:',l4
      if (abs(ip).gt.1e-3)
     &   write (0,*) 'warning : inner product in subroutine mirror:',ip
#     endif
C----
C---- adapt coordinates
C----
      p3=(p(1)-o(1))*v3(1)+(p(2)-o(2))*v3(2)+(p(3)-o(3))*v3(3)
      p4=(p(1)-o(1))*v4(1)+(p(2)-o(2))*v4(2)+(p(3)-o(3))*v4(3)
      pp(1)=o(1)+p3*v3(1)+p4*v4(1)
      pp(2)=o(2)+p3*v3(2)+p4*v4(2)
      pp(3)=o(3)+p3*v3(3)+p4*v4(3)

      x(1,n4)=2*pp(1)-p(1)
      x(2,n4)=2*pp(2)-p(2)
      x(3,n4)=2*pp(3)-p(3)
C----
C---- final check
C----
#     if defined(DEBUG)
      if (abs(clen-getlenxx(n1,n4,x)).gt.1e-3) then
         write (0,*) 'warning : different lengths in subroutine mirror:'
         write (0,*) 'old length:',clen
         write (0,*) 'new length:',getlenxx(n1,n4,x)
      endif
#     endif

      return

      end
      subroutine smooth(ndis,in,lbub,tag)

      implicit none

      include 'params.f'

      integer i,j,k,a,b,c,in(2,maxdis),ndis,tag(*),findpair,iout

      real lbub(2,maxdis),dum
C----
C---- first find pairs of pairs with one mutual entry, then see if the
C---- pair formed by the remaining two is an existing pair and therefore
C---- forms a closed triangle which can be smoothed
C----
      iout=ndis/10
      do i=1,ndis
         a=in(1,i)
         b=in(2,i)
         if (mod(i,iout).eq.0) write (6,100) 100.*real(i)/real(ndis)
C----
C---- here for case when in(1,i)=in(1,j)
C----
         do j=tag(in(1,i)),i-1
            c=in(2,j)
            k=findpair(b,c,in,tag)
            if (k.ne.0) then
               lbub(2,i)=min(lbub(2,i),lbub(2,j)+lbub(2,k))
               dum=max(lbub(1,i),lbub(1,j)-lbub(2,k))
               if (dum.eq.lbub(1,i)) lbub(1,i)=max(lbub(1,i),
     &              lbub(1,k)-lbub(2,j))
            endif
         enddo
C----
C---- here for case when in(2,i)=in(1,j)
C----
         do j=tag(in(2,i)),tag(in(2,i)+1)-1
            c=in(2,j)
            k=findpair(a,c,in,tag)
            if (k.ne.0) then
               lbub(2,i)=min(lbub(2,i),lbub(2,j)+lbub(2,k))
               dum=max(lbub(1,i),lbub(1,j)-lbub(2,k))
               if (dum.eq.lbub(1,i)) lbub(1,i)=max(lbub(1,i),
     &              lbub(1,k)-lbub(2,j))
            endif
         enddo
C----
C---- here for case when in(2,i)=in(2,j)
C----
         do j=tag(b+1),i-1
            if (b.eq.in(2,j)) then
               c=in(1,j)
               k=findpair(a,c,in,tag)
               if (k.ne.0) then
                  lbub(2,i)=min(lbub(2,i),lbub(2,j)+lbub(2,k))
                  dum=max(lbub(1,i),lbub(1,j)-lbub(2,k))
                  if (dum.eq.lbub(1,i)) lbub(1,i)=max(lbub(1,i),
     &                 lbub(1,k)-lbub(2,j))
               endif
            endif
         enddo
C----
C---- we don't have to worry about in(1,i)=in(2,j) since i > j
C----
      enddo

 100  format(f5.1,'% done')

      return
      
      end

      integer function findpair(k,l,in,tag)

      implicit none
      
      include 'params.f'

      integer i,n1,n2,in(2,maxdis),tag(*),k,l

      n1=k
      n2=l
      if (n1.lt.n2) call swap(n1,n2)

      findpair=0
      do i=tag(n1),tag(n1+1)-1
         if (n2.eq.in(2,i)) then
            findpair=i
            return
         endif
      enddo
      
      return
      end

